@@ -22,6 +22,7 @@
 #include <utils/memory-helpers.h>
 
 #include <Libkleo/Algorithm>
+#include <Libkleo/Compat>
 #include <Libkleo/Formatting>
 #include <Libkleo/KeyCache>
 #include <Libkleo/KeyGroupImportExport>
@@ -51,7 +52,6 @@
 #include <QEventLoop>
 #include <QProgressDialog>
 #include <QString>
-#include <QTextDocument> // for Qt::escape
 #include <QTreeView>
 #include <QWidget>
 
@@ -219,7 +219,7 @@ static QString format_ids(const std::vector<QString> &ids)
             escapedIds << id.toHtmlEscaped();
         }
     }
-    return escapedIds.join(QLatin1String("<br>"));
+    return escapedIds.join(QLatin1StringView("<br>"));
 }
 
 static QString make_tooltip(const std::vector<ImportResultData> &results)
@@ -312,7 +312,7 @@ static QString make_report(const std::vector<ImportResultData> &results, const s
     }
 
     if (!lines.empty()) {
-        lines.push_back(headerLine.subs(QLatin1String{"&nbsp;"}).toString());
+        lines.push_back(headerLine.subs(QLatin1StringView{"&nbsp;"}).toString());
     }
 
     if (groups.size() > 0) {
@@ -326,7 +326,7 @@ static QString make_report(const std::vector<ImportResultData> &results, const s
         lines.push_back(normalLine.subs(i18n("Updated groups:")).subs(updatedGroups).toString());
     }
 
-    return lines.join(QLatin1String{});
+    return lines.join(QLatin1StringView{});
 }
 
 static bool isImportFromSingleSource(const std::vector<ImportResultData> &res)
@@ -336,18 +336,18 @@ static bool isImportFromSingleSource(const std::vector<ImportResultData> &res)
 
 static QString make_message_report(const std::vector<ImportResultData> &res, const std::vector<ImportedGroup> &groups)
 {
-    QString report{QLatin1String{"<html>"}};
+    QString report{QLatin1StringView{"<html>"}};
     if (res.empty()) {
         report += i18n("No imports (should not happen, please report a bug).");
     } else {
         const QString title = isImportFromSingleSource(res) && !res.front().id.isEmpty() ? i18n("Detailed results of importing %1:", res.front().id)
                                                                                          : i18n("Detailed results of import:");
-        report += QLatin1String{"<p>"} + title + QLatin1String{"</p>"};
-        report += QLatin1String{"<p><table width=\"100%\">"};
+        report += QLatin1StringView{"<p>"} + title + QLatin1String{"</p>"};
+        report += QLatin1StringView{"<p><table width=\"100%\">"};
         report += make_report(res, groups);
-        report += QLatin1String{"</table></p>"};
+        report += QLatin1StringView{"</table></p>"};
     }
-    report += QLatin1String{"</html>"};
+    report += QLatin1StringView{"</html>"};
     return report;
 }
 
@@ -477,19 +477,25 @@ auto consolidatedAuditLogEntries(const std::vector<ImportResultData> &res)
         return s;
     };
     std::transform(res.cbegin(), res.cend(), std::back_inserter(auditLogs), extractAndAnnotateAuditLog);
-    return AuditLogEntry{auditLogs.join(QLatin1String{"<hr>"}), Error{}};
+    return AuditLogEntry{auditLogs.join(QLatin1StringView{"<hr>"}), Error{}};
 }
 }
 
 void ImportCertificatesCommand::Private::showDetails(const std::vector<ImportResultData> &res, const std::vector<ImportedGroup> &groups)
 {
     const auto singleOpenPGPImport = getSingleOpenPGPImport(res);
+
+    if (std::ranges::any_of(res, [](const auto &result) {
+            return result.result.numImported() > 0;
+        })) {
+        setImportResultProxyModel(res);
+    }
+
     if (!singleOpenPGPImport.isNull()) {
         if (showPleaseCertify(singleOpenPGPImport)) {
             return;
         }
     }
-    setImportResultProxyModel(res);
     MessageBox::information(parentWidgetOrView(), make_message_report(res, groups), consolidatedAuditLogEntries(res), i18n("Certificate Import Result"));
 }
 
@@ -707,7 +713,15 @@ void ImportCertificatesCommand::Private::processResults()
 
     handleOwnerTrust(results, parentWidgetOrView());
 
-    showDetails(results, importedGroups);
+    auto hasError = std::ranges::any_of(results, [](const auto &result) {
+        return importFailed(result);
+    });
+    auto allAreIrrelevant = std::ranges::all_of(results, [](const auto &result) {
+        return result.result.numConsidered() == 0 || (importFailed(result) && result.result.numConsidered() == 1);
+    });
+    if (!(hasError && allAreIrrelevant)) {
+        showDetails(results, importedGroups);
+    }
 
     auto tv = dynamic_cast<QTreeView *>(view());
     if (!tv) {
@@ -781,8 +795,17 @@ void ImportCertificatesCommand::Private::keyCacheUpdated()
     processResults();
 }
 
-static ImportedGroup storeGroup(const KeyGroup &group, const QString &id)
+static ImportedGroup storeGroup(const KeyGroup &group, const QString &id, QWidget *parent)
 {
+    if (Kleo::any_of(group.keys(), [](const auto &key) {
+            return !Kleo::keyHasEncrypt(key);
+        })) {
+        KMessageBox::information(parent,
+                                 xi18nc("@info",
+                                        "<para>The imported group</para><para><emphasis>%1</emphasis></para><para>contains certificates that cannot be used for encryption. "
+                                        "This may lead to unexpected results.</para>",
+                                        group.name()));
+    }
     const auto status = KeyCache::instance()->group(group.id()).isNull() ? ImportedGroup::Status::New : ImportedGroup::Status::Updated;
     if (status == ImportedGroup::Status::New) {
         KeyCache::mutableInstance()->insert(group);
@@ -801,8 +824,8 @@ void ImportCertificatesCommand::Private::importGroups()
         if (certificateImportSucceeded) {
             qCDebug(KLEOPATRA_LOG) << __func__ << "Importing groups from file" << path;
             const auto groups = readKeyGroups(path);
-            std::transform(std::begin(groups), std::end(groups), std::back_inserter(importedGroups), [path](const auto &group) {
-                return storeGroup(group, path);
+            std::transform(std::begin(groups), std::end(groups), std::back_inserter(importedGroups), [path, this](const auto &group) {
+                return storeGroup(group, path, parentWidgetOrView());
             });
         }
         increaseProgressValue();
