@@ -45,6 +45,7 @@
 #include <KLocalizedString>
 #include <KMessageBox>
 
+#include <QProgressDialog>
 #include <QRegularExpression>
 
 #include <algorithm>
@@ -58,6 +59,14 @@ using namespace Kleo::Dialogs;
 using namespace GpgME;
 using namespace QGpgME;
 
+namespace
+{
+struct KeyWithOrigin {
+    Key key;
+    Key::Origin origin;
+};
+}
+
 class LookupCertificatesCommand::Private : public ImportCertificatesCommand::Private
 {
     friend class ::Kleo::Commands::LookupCertificatesCommand;
@@ -119,17 +128,22 @@ private:
         }
     }
 
+    void cancelLookup();
+    void cancelJob(QPointer<Job> &job);
+
 private:
     GpgME::Protocol protocol = GpgME::UnknownProtocol;
     QString query;
     bool autoStartLookup = false;
     QPointer<LookupCertificatesDialog> dialog;
+    QPointer<QProgressDialog> progress;
     struct KeyListingVariables {
-        QPointer<KeyListJob> cms, openpgp;
-        QPointer<WKDLookupJob> wkdJob;
+        QPointer<Job> cms;
+        QPointer<Job> openpgp;
+        QPointer<Job> wkdJob;
         QString pattern;
         KeyListResult result;
-        std::vector<Key> keys;
+        std::vector<KeyWithOrigin> keys;
         int numKeysWithoutUserId = 0;
         std::set<std::string> wkdKeyFingerprints;
         QByteArray wkdKeyData;
@@ -247,6 +261,10 @@ void LookupCertificatesCommand::Private::createDialog()
     dialog = new LookupCertificatesDialog;
     applyWindowID(dialog);
     dialog->setAttribute(Qt::WA_DeleteOnClose);
+
+    const bool wkdOnly = !haveKeyserverConfigured() && !haveX509DirectoryServerConfigured();
+    dialog->setQueryMode(wkdOnly ? LookupCertificatesDialog::EmailQuery : LookupCertificatesDialog::AnyQuery);
+
     connect(dialog, &LookupCertificatesDialog::searchTextChanged, q, [this](const QString &text) {
         slotSearchTextChanged(text);
     });
@@ -275,13 +293,11 @@ void LookupCertificatesCommand::Private::slotSearchTextChanged(const QString &st
     // pressing return might trigger both search and dialog destruction (search focused and default key set)
     // On Windows, the dialog is then destroyed before this slot is called
     if (dialog) { // thus test
+        dialog->setOverlayText({});
         dialog->setPassive(true);
-        dialog->setCertificates(std::vector<Key>());
-        dialog->showInformation({});
+        dialog->setCertificates(std::vector<Key>(), {});
     }
 
-    query = str;
-
     keyListing.reset();
     keyListing.pattern = str;
 
@@ -290,21 +306,41 @@ void LookupCertificatesCommand::Private::slotSearchTextChanged(const QString &st
     }
 
     if (protocol != GpgME::CMS) {
-        static const QRegularExpression rx(QRegularExpression::anchoredPattern(QLatin1String("(?:0x|0X)?[0-9a-fA-F]{6,}")));
-        if (rx.match(query).hasMatch() && !str.startsWith(QLatin1String("0x"), Qt::CaseInsensitive)) {
-            qCDebug(KLEOPATRA_LOG) << "Adding 0x prefix to query";
-            startKeyListJob(OpenPGP, QStringLiteral("0x") + str);
+        static const QRegularExpression rx(QRegularExpression::anchoredPattern(QLatin1StringView("[0-9a-fA-F]{6,}")));
+        if (rx.match(str).hasMatch()) {
+            qCDebug(KLEOPATRA_LOG) << "Adding 0x prefix to query" << str;
+            startKeyListJob(OpenPGP, QLatin1StringView{"0x"} + str);
         } else {
             startKeyListJob(OpenPGP, str);
-            if (str.contains(QLatin1Char{'@'}) && !searchTextToEmailAddress(str).isEmpty()) {
-                startWKDLookupJob(str);
-            }
         }
+        if (str.contains(QLatin1Char{'@'}) && !searchTextToEmailAddress(str).isEmpty()) {
+            startWKDLookupJob(str);
+        }
+    }
+
+    const auto jobCount = int(!keyListing.cms.isNull()) + int(!keyListing.openpgp.isNull()) + int(!keyListing.wkdJob.isNull());
+    if (jobCount > 0) {
+        progress = new QProgressDialog{dialog};
+        progress->setAttribute(Qt::WA_DeleteOnClose);
+        progress->setLabelText(i18nc("@info", "Searching for matching certificates ..."));
+        progress->setMaximum(jobCount);
+        progress->setMinimumDuration(0);
+        progress->setValue(0);
+        connect(progress, &QProgressDialog::canceled, q, [this]() {
+            cancelLookup();
+        });
     }
 }
 
 void LookupCertificatesCommand::Private::startKeyListJob(GpgME::Protocol proto, const QString &str)
 {
+    if ((proto == GpgME::OpenPGP) && !haveKeyserverConfigured()) {
+        // avoid starting an OpenPGP key server lookup if key server usage has been disabled;
+        // for S/MIME we start the job regardless of configured directory servers to account for
+        // dirmngr knowing better than our check for directory servers
+        return;
+    }
+
     KeyListJob *const klj = createKeyListJob(proto);
     if (!klj) {
         return;
@@ -343,7 +379,9 @@ void LookupCertificatesCommand::Private::startWKDLookupJob(const QString &str)
 
 void LookupCertificatesCommand::Private::slotNextKey(const Key &key)
 {
-    if (!key.primaryFingerprint()) {
+    if (key.isNull()) {
+        qCDebug(KLEOPATRA_LOG) << __func__ << "ignoring null key";
+    } else if (!key.primaryFingerprint()) {
         qCDebug(KLEOPATRA_LOG) << __func__ << "ignoring key without fingerprint" << key;
         if (q->sender() == keyListing.cms) {
             keyListing.cmsKeysHaveNoFingerprints = true;
@@ -355,7 +393,7 @@ void LookupCertificatesCommand::Private::slotNextKey(const Key &key)
         keyListing.numKeysWithoutUserId++;
     } else {
         qCDebug(KLEOPATRA_LOG) << __func__ << "got key" << key;
-        keyListing.keys.push_back(key);
+        keyListing.keys.push_back({key, Key::OriginKS});
     }
 }
 
@@ -405,7 +443,9 @@ void LookupCertificatesCommand::Private::slotWKDLookupResult(const WKDLookupResu
     if (!keys.empty()) {
         keyListing.wkdKeyData = QByteArray::fromStdString(result.keyData().toString());
         keyListing.wkdSource = QString::fromStdString(result.source());
-        std::copy(std::begin(keys), std::end(keys), std::back_inserter(keyListing.keys));
+        for (const auto &key : keys) {
+            keyListing.keys.push_back({key, Key::OriginWKD});
+        }
         // remember the keys retrieved via WKD for import
         std::transform(std::begin(keys),
                        std::end(keys),
@@ -448,12 +488,18 @@ void showKeysWithoutFingerprintsNotification(QWidget *parent, GpgME::Protocol pr
 
 void LookupCertificatesCommand::Private::tryToFinishKeyLookup()
 {
+    if (progress) {
+        progress->setValue(progress->value() + 1);
+    }
     if (keyListing.cms || keyListing.openpgp || keyListing.wkdJob) {
         // still waiting for jobs to complete
         return;
     }
+    if (progress) {
+        progress->setValue(progress->maximum());
+    }
 
-    if (keyListing.result.error() && !keyListing.result.error().isCanceled()) {
+    if (keyListing.result.error() && !keyListing.result.error().isCanceled() && (keyListing.result.error().code() != GPG_ERR_NOT_FOUND)) {
         showError(dialog, keyListing.result);
     }
 
@@ -470,12 +516,21 @@ void LookupCertificatesCommand::Private::tryToFinishKeyLookup()
 
     if (dialog) {
         dialog->setPassive(false);
-        dialog->setCertificates(keyListing.keys);
+
+        std::sort(keyListing.keys.begin(), keyListing.keys.end(), [](const auto &lhs, const auto &rhs) {
+            return qstricmp(lhs.key.primaryFingerprint(), rhs.key.primaryFingerprint());
+        });
+        std::vector<Key> keys;
+        std::vector<Key::Origin> origins;
+        for (const auto &pair : keyListing.keys) {
+            keys.push_back(pair.key), origins.push_back(pair.origin);
+        }
+        dialog->setCertificates(keys, origins);
+        if (keyListing.keys.size() == 0) {
+            dialog->setOverlayText(i18nc("@info", "No certificates found"));
+        }
         if (keyListing.numKeysWithoutUserId > 0) {
-            dialog->showInformation(i18ncp("@info",
-                                           "One certificate without name and email address was ignored.",
-                                           "%1 certificates without name and email address were ignored.",
-                                           keyListing.numKeysWithoutUserId));
+            qCDebug(KLEOPATRA_LOG) << keyListing.numKeysWithoutUserId << "certificates without user IDs were ignored";
         }
     } else {
         finished();
@@ -507,7 +562,7 @@ void LookupCertificatesCommand::Private::slotImportRequested(const std::vector<K
     setWaitForMoreJobs(true);
     if (!wkdKeys.empty()) {
         // set an import filter, so that only user IDs matching the email address used for the WKD lookup are imported
-        const QString importFilter = QLatin1String{"keep-uid=mbox = "} + searchTextToEmailAddress(keyListing.pattern);
+        const QString importFilter = QLatin1StringView{"keep-uid=mbox = "} + searchTextToEmailAddress(keyListing.pattern);
         startImport(OpenPGP, keyListing.wkdKeyData, keyListing.wkdSource, {importFilter, Key::OriginWKD, keyListing.wkdSource});
     }
     if (!pgp.empty()) {
@@ -532,6 +587,28 @@ void LookupCertificatesCommand::Private::slotDetailsRequested(const Key &key)
     cmd->start();
 }
 
+void LookupCertificatesCommand::Private::cancelLookup()
+{
+    cancelJob(keyListing.cms);
+    cancelJob(keyListing.openpgp);
+    cancelJob(keyListing.wkdJob);
+
+    if (dialog) {
+        dialog->setPassive(false);
+    } else {
+        finished();
+    }
+}
+
+void LookupCertificatesCommand::Private::cancelJob(QPointer<Job> &job)
+{
+    if (job) {
+        disconnect(job.data(), nullptr, q, nullptr);
+        job->slotCancel();
+        job.clear();
+    }
+}
+
 void LookupCertificatesCommand::doCancel()
 {
     ImportCertificatesCommand::doCancel();
@@ -569,10 +646,9 @@ void LookupCertificatesCommand::Private::showResult(QWidget *parent, const KeyLi
 
 bool LookupCertificatesCommand::Private::checkConfig() const
 {
-    const bool haveOrDontNeedOpenPGPServer = haveKeyserverConfigured() || (protocol == GpgME::CMS);
-    const bool haveOrDontNeedCMSServer = haveX509DirectoryServerConfigured() || (protocol == GpgME::OpenPGP);
-    const bool ok = haveOrDontNeedOpenPGPServer || haveOrDontNeedCMSServer;
-    if (!ok)
+    // unless CMS-only lookup is requested we always try a lookup via WKD
+    const bool ok = (protocol != GpgME::CMS) || haveX509DirectoryServerConfigured();
+    if (!ok) {
         information(xi18nc("@info",
                            "<para>You do not have any directory servers configured.</para>"
                            "<para>You need to configure at least one directory server to "
@@ -580,6 +656,7 @@ bool LookupCertificatesCommand::Private::checkConfig() const
                            "<para>You can configure directory servers here: "
                            "<interface>Settings->Configure Kleopatra</interface>.</para>"),
                     i18nc("@title", "No Directory Servers Configured"));
+    }
     return ok;
 }
 
