@@ -23,14 +23,16 @@
 
 #include "commands/decryptverifyfilescommand.h"
 
-#include <Libkleo/GnuPG>
 #include <utils/archivedefinition.h>
 #include <utils/input.h>
 #include <utils/kleo_assert.h>
 #include <utils/output.h>
+#include <utils/overwritedialog.h>
 #include <utils/path-helper.h>
 
+#include <Libkleo/Algorithm>
 #include <Libkleo/Classify>
+#include <Libkleo/GnuPG>
 
 #ifndef Q_OS_WIN
 #include <KIO/CopyJob>
@@ -167,6 +169,7 @@ void AutoDecryptVerifyFilesController::Private::exec()
     DecryptVerifyFilesDialog dialog{coll};
     m_dialog = &dialog;
     m_dialog->setOutputLocation(heuristicBaseDirectory(m_passedFiles));
+    q->applyWindowID(m_dialog);
 
     QTimer::singleShot(0, q, SLOT(schedule()));
     const auto result = m_dialog->exec();
@@ -176,9 +179,13 @@ void AutoDecryptVerifyFilesController::Private::exec()
         // Without workdir there is nothing to move.
         const QDir workdir(m_workDir->path());
         const QDir outDir(m_dialog->outputLocation());
-        bool overWriteAll = false;
         qCDebug(KLEOPATRA_LOG) << workdir.entryList(QDir::Files | QDir::Dirs | QDir::NoDotAndDotDot);
-        for (const QFileInfo &fi : workdir.entryInfoList(QDir::Files | QDir::Dirs | QDir::NoDotAndDotDot)) {
+        const auto filesAndFoldersToMove = workdir.entryInfoList(QDir::Files | QDir::Dirs | QDir::NoDotAndDotDot);
+        const auto fileCount = Kleo::count_if(filesAndFoldersToMove, [](const auto &fi) {
+            return !fi.isDir();
+        });
+        OverwritePolicy overwritePolicy{m_dialog, fileCount > 1 ? OverwritePolicy::MultipleFiles : OverwritePolicy::Options{}};
+        for (const QFileInfo &fi : filesAndFoldersToMove) {
             const auto inpath = fi.absoluteFilePath();
 
             if (fi.isDir()) {
@@ -245,31 +252,27 @@ void AutoDecryptVerifyFilesController::Private::exec()
                     outFileName = embeddedFileName;
                 }
             }
-            const auto outpath = outDir.absoluteFilePath(outFileName);
+            auto outpath = outDir.absoluteFilePath(outFileName);
             qCDebug(KLEOPATRA_LOG) << "Moving " << inpath << " to " << outpath;
             const QFileInfo ofi(outpath);
             if (ofi.exists()) {
-                int sel = KMessageBox::Cancel;
-                if (!overWriteAll) {
-                    sel = KMessageBox::questionTwoActionsCancel(m_dialog,
-                                                                i18n("The file <b>%1</b> already exists.\n"
-                                                                     "Overwrite?",
-                                                                     outpath),
-                                                                i18n("Overwrite Existing File?"),
-                                                                KStandardGuiItem::overwrite(),
-                                                                KGuiItem(i18n("Overwrite All")),
-                                                                KStandardGuiItem::cancel());
-                }
-                if (sel == KMessageBox::Cancel) {
-                    qCDebug(KLEOPATRA_LOG) << "Overwriting canceled for: " << outpath;
-                    continue;
-                }
-                if (sel == KMessageBox::ButtonCode::SecondaryAction) { // Overwrite All
-                    overWriteAll = true;
-                }
-                if (!QFile::remove(outpath)) {
-                    reportError(makeGnuPGError(GPG_ERR_GENERAL), xi18n("Failed to delete <filename>%1</filename>.", outpath));
+                const auto newPath = overwritePolicy.obtainOverwritePermission(outpath);
+                if (newPath.isEmpty()) {
+                    if (overwritePolicy.policy() == OverwritePolicy::Cancel) {
+                        qCDebug(KLEOPATRA_LOG) << "Overwriting canceled for: " << outpath;
+                        break;
+                    }
+                    // else Skip
                     continue;
+                } else if (newPath == outpath) {
+                    // overwrite existing file
+                    if (!QFile::remove(outpath)) {
+                        reportError(makeGnuPGError(GPG_ERR_GENERAL), xi18n("Failed to delete <filename>%1</filename>.", outpath));
+                        continue;
+                    }
+                } else {
+                    // use new name for file
+                    outpath = newPath;
                 }
             }
             if (!QFile::rename(inpath, outpath)) {
@@ -404,8 +407,18 @@ std::vector<std::shared_ptr<Task>> AutoDecryptVerifyFilesController::Private::bu
             } else {
                 qCDebug(KLEOPATRA_LOG) << "Detached verify: " << cFile.fileName << " Data: " << signedDataFileName;
                 std::shared_ptr<VerifyDetachedTask> t(new VerifyDetachedTask);
+#if QGPGME_FILE_JOBS_SUPPORT_DIRECT_FILE_IO
+                if (cFile.protocol == GpgME::OpenPGP) {
+                    t->setSignatureFile(cFile.fileName);
+                    t->setSignedFile(signedDataFileName);
+                } else {
+                    t->setInput(Input::createFromFile(cFile.fileName));
+                    t->setSignedData(Input::createFromFile(signedDataFileName));
+                }
+#else
                 t->setInput(Input::createFromFile(cFile.fileName));
                 t->setSignedData(Input::createFromFile(signedDataFileName));
+#endif
                 t->setProtocol(cFile.protocol);
                 tasks.push_back(t);
             }
@@ -427,8 +440,18 @@ std::vector<std::shared_ptr<Task>> AutoDecryptVerifyFilesController::Private::bu
                     }
                     foundSig = true;
                     std::shared_ptr<VerifyDetachedTask> t(new VerifyDetachedTask);
+#if QGPGME_FILE_JOBS_SUPPORT_DIRECT_FILE_IO
+                    if (cFile.protocol == GpgME::OpenPGP) {
+                        t->setSignatureFile(sig);
+                        t->setSignedFile(cFile.fileName);
+                    } else {
+                        t->setInput(Input::createFromFile(sig));
+                        t->setSignedData(Input::createFromFile(cFile.fileName));
+                    }
+#else
                     t->setInput(Input::createFromFile(sig));
                     t->setSignedData(Input::createFromFile(cFile.fileName));
+#endif
                     t->setProtocol(proto);
                     tasks.push_back(t);
                 }
@@ -440,7 +463,14 @@ std::vector<std::shared_ptr<Task>> AutoDecryptVerifyFilesController::Private::bu
         } else {
             const FileOperationsPreferences fileOpSettings;
             // Any Message type so we have input and output.
-            const auto input = Input::createFromFile(cFile.fileName);
+            std::shared_ptr<Input> input;
+#if QGPGME_FILE_JOBS_SUPPORT_DIRECT_FILE_IO
+            if (cFile.protocol != GpgME::OpenPGP) {
+                input = Input::createFromFile(cFile.fileName);
+            }
+#else
+            input = Input::createFromFile(cFile.fileName);
+#endif
 
             std::shared_ptr<ArchiveDefinition> ad;
             if (fileOpSettings.autoExtractArchives()) {
@@ -465,14 +495,22 @@ std::vector<std::shared_ptr<Task>> AutoDecryptVerifyFilesController::Private::bu
             const auto wd = QDir(m_workDir->path());
 
             std::shared_ptr<Output> output;
+            QString outputFilePath;
             if (ad) {
-                if ((ad->id() == QLatin1String{"tar"}) && archiveJobsCanBeUsed(cFile.protocol)) {
+                if ((ad->id() == QLatin1StringView{"tar"}) && archiveJobsCanBeUsed(cFile.protocol)) {
                     // we don't need an output
                 } else {
                     output = ad->createOutputFromUnpackCommand(cFile.protocol, ad->stripExtension(cFile.protocol, cFile.baseName), wd);
                 }
             } else {
-                output = Output::createFromFile(wd.absoluteFilePath(outputFileName(fi.fileName())), false);
+                outputFilePath = wd.absoluteFilePath(outputFileName(fi.fileName()));
+#if QGPGME_FILE_JOBS_SUPPORT_DIRECT_FILE_IO
+                if (cFile.protocol != GpgME::OpenPGP) {
+                    output = Output::createFromFile(outputFilePath, false);
+                }
+#else
+                output = Output::createFromFile(outputFilePath, false);
+#endif
             }
 
             // If this might be opaque CMS signature, then try that. We already handled
@@ -482,7 +520,9 @@ std::vector<std::shared_ptr<Task>> AutoDecryptVerifyFilesController::Private::bu
             if (isOpaqueSignature(cFile.classification) || isCMSOpaqueSignature) {
                 qCDebug(KLEOPATRA_LOG) << "creating a VerifyOpaqueTask";
                 std::shared_ptr<VerifyOpaqueTask> t(new VerifyOpaqueTask);
-                t->setInput(input);
+                if (input) {
+                    t->setInput(input);
+                }
                 if (output) {
                     t->setOutput(output);
                 }
@@ -497,6 +537,11 @@ std::vector<std::shared_ptr<Task>> AutoDecryptVerifyFilesController::Private::bu
                         const auto baseFileName = QFileInfo{ad->stripExtension(cFile.protocol, cFile.baseName)}.fileName();
                         t->setOutputDirectory(QDir{m_workDir->path()}.filePath(baseFileName));
                     }
+#if QGPGME_FILE_JOBS_SUPPORT_DIRECT_FILE_IO
+                } else if (cFile.protocol == GpgME::OpenPGP) {
+                    t->setInputFile(cFile.fileName);
+                    t->setOutputFile(outputFilePath);
+#endif
                 }
                 tasks.push_back(t);
             } else {
@@ -505,7 +550,9 @@ std::vector<std::shared_ptr<Task>> AutoDecryptVerifyFilesController::Private::bu
                 // an encrypted message is also signed.
                 qCDebug(KLEOPATRA_LOG) << "creating a DecryptVerifyTask";
                 std::shared_ptr<DecryptVerifyTask> t(new DecryptVerifyTask);
-                t->setInput(input);
+                if (input) {
+                    t->setInput(input);
+                }
                 if (output) {
                     t->setOutput(output);
                 }
@@ -520,6 +567,11 @@ std::vector<std::shared_ptr<Task>> AutoDecryptVerifyFilesController::Private::bu
                         const auto baseFileName = QFileInfo{ad->stripExtension(cFile.protocol, cFile.baseName)}.fileName();
                         t->setOutputDirectory(QDir{m_workDir->path()}.filePath(baseFileName));
                     }
+#if QGPGME_FILE_JOBS_SUPPORT_DIRECT_FILE_IO
+                } else if (cFile.protocol == GpgME::OpenPGP) {
+                    t->setInputFile(cFile.fileName);
+                    t->setOutputFile(outputFilePath);
+#endif
                 }
                 cFile.output = output;
                 tasks.push_back(t);
