@@ -71,9 +71,14 @@ public Q_SLOTS:
 protected:
     std::shared_ptr<DecryptVerifyResult> fromDecryptResult(const GpgME::DecryptionResult &dr, const QByteArray &plaintext, const AuditLogEntry &auditLog);
     std::shared_ptr<DecryptVerifyResult> fromDecryptResult(const GpgME::Error &err, const QString &details, const AuditLogEntry &auditLog);
-    std::shared_ptr<DecryptVerifyResult> fromDecryptVerifyResult(const GpgME::DecryptionResult &dr, const GpgME::VerificationResult &vr, const QByteArray &plaintext, const QString &fileName, const AuditLogEntry &auditLog);
+    std::shared_ptr<DecryptVerifyResult> fromDecryptVerifyResult(const GpgME::DecryptionResult &dr,
+                                                                 const GpgME::VerificationResult &vr,
+                                                                 const QByteArray &plaintext,
+                                                                 const QString &fileName,
+                                                                 const AuditLogEntry &auditLog);
     std::shared_ptr<DecryptVerifyResult> fromDecryptVerifyResult(const GpgME::Error &err, const QString &what, const AuditLogEntry &auditLog);
-    std::shared_ptr<DecryptVerifyResult> fromVerifyOpaqueResult(const GpgME::VerificationResult &vr, const QByteArray &plaintext, const AuditLogEntry &auditLog);
+    std::shared_ptr<DecryptVerifyResult>
+    fromVerifyOpaqueResult(const GpgME::VerificationResult &vr, const QByteArray &plaintext, const AuditLogEntry &auditLog);
     std::shared_ptr<DecryptVerifyResult> fromVerifyOpaqueResult(const GpgME::Error &err, const QString &details, const AuditLogEntry &auditLog);
     std::shared_ptr<DecryptVerifyResult> fromVerifyDetachedResult(const GpgME::VerificationResult &vr, const AuditLogEntry &auditLog);
     std::shared_ptr<DecryptVerifyResult> fromVerifyDetachedResult(const GpgME::Error &err, const QString &details, const AuditLogEntry &auditLog);
@@ -128,6 +133,9 @@ public:
     void setInput(const std::shared_ptr<Input> &input);
     void setSignedData(const std::shared_ptr<Input> &signedData);
 
+    void setSignatureFile(const QString &path);
+    void setSignedFile(const QString &path);
+
     void setProtocol(GpgME::Protocol prot);
     void autodetectProtocolFromInput() override;
 
@@ -159,13 +167,16 @@ public:
     void setOutput(const std::shared_ptr<Output> &output);
 
     void setProtocol(GpgME::Protocol prot);
-    void autodetectProtocolFromInput()override;
+    void autodetectProtocolFromInput() override;
 
     QString label() const override;
     GpgME::Protocol protocol() const override;
 
     void setExtractArchive(bool extract);
     void setInputFile(const QString &path);
+    // for files
+    void setOutputFile(const QString &path);
+    // for archives
     void setOutputDirectory(const QString &directory);
 
     QString inputLabel() const override;
@@ -201,6 +212,9 @@ public:
     void setIgnoreMDCError(bool value);
     void setExtractArchive(bool extract);
     void setInputFile(const QString &path);
+    // for files
+    void setOutputFile(const QString &path);
+    // for archives
     void setOutputDirectory(const QString &directory);
 
     QString inputLabel() const override;
@@ -219,6 +233,7 @@ private:
 class DecryptVerifyResult : public Task::Result
 {
     friend class ::Kleo::Crypto::AbstractDecryptVerifyTask;
+
 public:
     class SenderInfo;
 
@@ -229,6 +244,7 @@ public:
     VisualCode code() const override;
     AuditLogEntry auditLog() const override;
     QPointer<Task> parentTask() const override;
+    Task::Result::ContentType viewableContentType() const override;
 
     GpgME::VerificationResult verificationResult() const;
     GpgME::DecryptionResult decryptionResult() const;
@@ -258,4 +274,3 @@ private:
 };
 }
 }
-
