@@ -11,6 +11,7 @@
 
 #include "certifywidget.h"
 
+#include "dialogs/animatedexpander.h"
 #include "view/infofield.h"
 #include <utils/accessibility.h>
 #include <utils/expiration.h>
@@ -34,8 +35,8 @@
 #include <Libkleo/KeyCache>
 #include <Libkleo/KeyHelpers>
 #include <Libkleo/KeySelectionCombo>
-#include <Libkleo/NavigatableTreeWidget>
 #include <Libkleo/Predicates>
+#include <Libkleo/TreeWidget>
 
 #include <QGpgME/ChangeOwnerTrustJob>
 #include <QGpgME/Protocol>
@@ -68,120 +69,6 @@ static QDebug operator<<(QDebug s, const GpgME::UserID &userID)
 namespace
 {
 
-// Maybe move this in its own file
-// based on code from StackOverflow
-class AnimatedExpander : public QWidget
-{
-    Q_OBJECT
-public:
-    explicit AnimatedExpander(const QString &title, const QString &accessibleTitle = {}, QWidget *parent = nullptr);
-    void setContentLayout(QLayout *contentLayout);
-    bool isExpanded() const;
-    void setExpanded(bool expanded);
-
-private:
-    static const int animationDuration = 300;
-
-    QGridLayout mainLayout;
-    QToolButton toggleButton;
-    QFrame headerLine;
-    QParallelAnimationGroup toggleAnimation;
-    QWidget contentArea;
-};
-
-AnimatedExpander::AnimatedExpander(const QString &title, const QString &accessibleTitle, QWidget *parent)
-    : QWidget{parent}
-{
-#ifdef Q_OS_WIN
-    // draw dotted focus frame if button has focus; otherwise, draw invisible frame using background color
-    toggleButton.setStyleSheet(
-        QStringLiteral("QToolButton { border: 1px solid palette(window); }"
-                       "QToolButton:focus { border: 1px dotted palette(window-text); }"));
-#else
-    // this works with Breeze style because Breeze draws the focus frame when drawing CE_ToolButtonLabel
-    // while the Windows styles (and Qt's common base style) draw the focus frame before drawing CE_ToolButtonLabel
-    toggleButton.setStyleSheet(QStringLiteral("QToolButton { border: none; }"));
-#endif
-    toggleButton.setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
-    toggleButton.setArrowType(Qt::ArrowType::RightArrow);
-    toggleButton.setText(title);
-    if (!accessibleTitle.isEmpty()) {
-        toggleButton.setAccessibleName(accessibleTitle);
-    }
-    toggleButton.setCheckable(true);
-    toggleButton.setChecked(false);
-
-    headerLine.setFrameShape(QFrame::HLine);
-    headerLine.setFrameShadow(QFrame::Sunken);
-    headerLine.setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);
-
-    contentArea.setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
-
-    // start out collapsed
-    contentArea.setMaximumHeight(0);
-    contentArea.setMinimumHeight(0);
-    contentArea.setVisible(false);
-
-    // let the entire widget grow and shrink with its content
-    toggleAnimation.addAnimation(new QPropertyAnimation(this, "minimumHeight"));
-    toggleAnimation.addAnimation(new QPropertyAnimation(this, "maximumHeight"));
-    toggleAnimation.addAnimation(new QPropertyAnimation(&contentArea, "maximumHeight"));
-
-    mainLayout.setVerticalSpacing(0);
-    mainLayout.setContentsMargins(0, 0, 0, 0);
-    int row = 0;
-    mainLayout.addWidget(&toggleButton, row, 0, 1, 1, Qt::AlignLeft);
-    mainLayout.addWidget(&headerLine, row++, 2, 1, 1);
-    mainLayout.addWidget(&contentArea, row, 0, 1, 3);
-    setLayout(&mainLayout);
-    connect(&toggleButton, &QToolButton::toggled, this, [this](const bool checked) {
-        if (checked) {
-            // make the content visible when expanding starts
-            contentArea.setVisible(true);
-        }
-        // use instant animation if widget isn't visible (e.g. before widget is shown)
-        const int duration = isVisible() ? animationDuration : 0;
-        // update the size of the content area
-        const auto collapsedHeight = sizeHint().height() - contentArea.maximumHeight();
-        const auto contentHeight = contentArea.layout()->sizeHint().height();
-        for (int i = 0; i < toggleAnimation.animationCount() - 1; ++i) {
-            auto expanderAnimation = static_cast<QPropertyAnimation *>(toggleAnimation.animationAt(i));
-            expanderAnimation->setDuration(duration);
-            expanderAnimation->setStartValue(collapsedHeight);
-            expanderAnimation->setEndValue(collapsedHeight + contentHeight);
-        }
-        auto contentAnimation = static_cast<QPropertyAnimation *>(toggleAnimation.animationAt(toggleAnimation.animationCount() - 1));
-        contentAnimation->setDuration(duration);
-        contentAnimation->setStartValue(0);
-        contentAnimation->setEndValue(contentHeight);
-        toggleButton.setArrowType(checked ? Qt::ArrowType::DownArrow : Qt::ArrowType::RightArrow);
-        toggleAnimation.setDirection(checked ? QAbstractAnimation::Forward : QAbstractAnimation::Backward);
-        toggleAnimation.start();
-    });
-    connect(&toggleAnimation, &QAbstractAnimation::finished, this, [this]() {
-        // hide the content area when it is fully collapsed
-        if (!toggleButton.isChecked()) {
-            contentArea.setVisible(false);
-        }
-    });
-}
-
-void AnimatedExpander::setContentLayout(QLayout *contentLayout)
-{
-    delete contentArea.layout();
-    contentArea.setLayout(contentLayout);
-}
-
-bool AnimatedExpander::isExpanded() const
-{
-    return toggleButton.isChecked();
-}
-
-void AnimatedExpander::setExpanded(bool expanded)
-{
-    toggleButton.setChecked(expanded);
-}
-
 class SecKeyFilter : public DefaultKeyFilter
 {
 public:
@@ -221,18 +108,18 @@ auto checkBoxSize(const QCheckBox *checkBox)
     return checkBox->style()->sizeFromContents(QStyle::CT_CheckBox, &opt, QSize(), checkBox);
 }
 
-class TreeWidget : public NavigatableTreeWidget
+class TreeWidgetInternal : public TreeWidget
 {
     Q_OBJECT
 public:
-    using NavigatableTreeWidget::NavigatableTreeWidget;
+    using TreeWidget::TreeWidget;
 
 protected:
     void focusInEvent(QFocusEvent *event) override
     {
-        NavigatableTreeWidget::focusInEvent(event);
+        TreeWidget::focusInEvent(event);
         // queue the invokation, so that it happens after the widget itself got focus
-        QMetaObject::invokeMethod(this, &TreeWidget::forceAccessibleFocusEventForCurrentItem, Qt::QueuedConnection);
+        QMetaObject::invokeMethod(this, &TreeWidgetInternal::forceAccessibleFocusEventForCurrentItem, Qt::QueuedConnection);
     }
 
     bool edit(const QModelIndex &index, EditTrigger trigger, QEvent *event) override
@@ -241,10 +128,10 @@ protected:
             const auto *const keyEvent = static_cast<QKeyEvent *>(event);
             if (keyEvent->key() == Qt::Key_Space || keyEvent->key() == Qt::Key_Select) {
                 // toggle checked state regardless of the index's column
-                return NavigatableTreeWidget::edit(index.siblingAtColumn(0), trigger, event);
+                return TreeWidget::edit(index.siblingAtColumn(0), trigger, event);
             }
         }
-        return NavigatableTreeWidget::edit(index, trigger, event);
+        return TreeWidget::edit(index, trigger, event);
     }
 
 private:
@@ -524,7 +411,7 @@ public:
 
     void saveConfig()
     {
-        KConfigGroup conf{KSharedConfig::openConfig(), QLatin1String("CertifySettings")};
+        KConfigGroup conf{KSharedConfig::openConfig(), QLatin1StringView("CertifySettings")};
         if (!secKey().isNull()) {
             conf.writeEntry("LastKey", secKey().primaryFingerprint());
         }
@@ -970,7 +857,7 @@ public:
     KeySelectionCombo *mSecKeySelect = nullptr;
     KMessageWidget *mMissingOwnerTrustInfo = nullptr;
     KMessageWidget *mBadCertificatesInfo = nullptr;
-    NavigatableTreeWidget *userIdListView = nullptr;
+    TreeWidget *userIdListView = nullptr;
     AnimatedExpander *mAdvancedOptionsExpander = nullptr;
     QCheckBox *mExportCB = nullptr;
     QCheckBox *mPublishCB = nullptr;
