@@ -11,13 +11,15 @@
 
 #include "lookupcertificatesdialog.h"
 
-#include "view/keytreeview.h"
+#include <view/keytreeview.h>
+#include <view/textoverlay.h>
+
+#include <kleopatra_debug.h>
 
 #include <Libkleo/KeyListModel>
 
 #include <KConfigGroup>
 #include <KLocalizedString>
-#include <KMessageWidget>
 #include <KSeparator>
 #include <KSharedConfig>
 
@@ -26,6 +28,8 @@
 #include <QLabel>
 #include <QLineEdit>
 #include <QPushButton>
+#include <QRegularExpression>
+#include <QRegularExpressionValidator>
 #include <QTreeView>
 #include <QVBoxLayout>
 
@@ -35,8 +39,6 @@ using namespace Kleo;
 using namespace Kleo::Dialogs;
 using namespace GpgME;
 
-static const int minimalSearchTextLength = 1;
-
 class LookupCertificatesDialog::Private
 {
     friend class ::Kleo::Dialogs::LookupCertificatesDialog;
@@ -57,7 +59,7 @@ private:
     }
     void slotSearchClicked()
     {
-        Q_EMIT q->searchTextChanged(ui.findED->text());
+        Q_EMIT q->searchTextChanged(searchText());
     }
     void slotDetailsClicked()
     {
@@ -96,19 +98,26 @@ private:
         return ui.resultTV->selectedKeys().size();
     }
 
+    QValidator *queryValidator();
+    void updateQueryMode();
+
 private:
+    QueryMode queryMode = AnyQuery;
     bool passive;
+    QValidator *anyQueryValidator = nullptr;
+    QValidator *emailQueryValidator = nullptr;
 
     struct Ui {
+        QLabel *guidanceLabel;
         QLabel *findLB;
         QLineEdit *findED;
         QPushButton *findPB;
         Kleo::KeyTreeView *resultTV;
+        TextOverlay *overlay;
         QPushButton *selectAllPB;
         QPushButton *deselectAllPB;
         QPushButton *detailsPB;
         QPushButton *saveAsPB;
-        KMessageWidget *messageWidget;
         QDialogButtonBox *buttonBox;
 
         void setupUi(QDialog *dialog)
@@ -117,6 +126,10 @@ private:
             auto gridLayout = new QGridLayout{};
 
             int row = 0;
+            guidanceLabel = new QLabel{dialog};
+            gridLayout->addWidget(guidanceLabel, row, 0, 1, 3);
+
+            row++;
             findLB = new QLabel{i18n("Find:"), dialog};
             gridLayout->addWidget(findLB, row, 0, 1, 1);
 
@@ -135,6 +148,8 @@ private:
             resultTV = new Kleo::KeyTreeView(dialog);
             resultTV->setEnabled(true);
             resultTV->setMinimumSize(QSize(400, 0));
+            overlay = new TextOverlay{resultTV, dialog};
+            overlay->hide();
             gridLayout->addWidget(resultTV, row, 0, 1, 2);
 
             auto buttonLayout = new QVBoxLayout{};
@@ -163,12 +178,6 @@ private:
 
             gridLayout->addLayout(buttonLayout, row, 2, 1, 1);
 
-            row++;
-            messageWidget = new KMessageWidget{dialog};
-            messageWidget->setMessageType(KMessageWidget::Information);
-            messageWidget->setVisible(false);
-            gridLayout->addWidget(messageWidget, row, 0, 1, 3);
-
             verticalLayout->addLayout(gridLayout);
 
             buttonBox = new QDialogButtonBox{dialog};
@@ -201,7 +210,7 @@ private:
             importPB()->setText(i18n("Import"));
             importPB()->setEnabled(false);
 
-            connect(resultTV->view(), SIGNAL(doubleClicked(QModelIndex)), importPB(), SLOT(animateClick()));
+            connect(resultTV->view(), SIGNAL(doubleClicked(QModelIndex)), q, SLOT(slotDetailsClicked()));
 
             findED->setFocus();
 
@@ -226,6 +235,7 @@ LookupCertificatesDialog::Private::Private(LookupCertificatesDialog *qq)
     , ui(q)
 {
     connect(ui.resultTV->view()->selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)), q, SLOT(slotSelectionChanged()));
+    updateQueryMode();
 }
 
 LookupCertificatesDialog::Private::~Private()
@@ -249,13 +259,53 @@ void LookupCertificatesDialog::Private::writeConfig()
 {
     KConfigGroup configGroup(KSharedConfig::openStateConfig(), QStringLiteral("LookupCertificatesDialog"));
     configGroup.writeEntry("Size", q->size());
-
-    KConfigGroup resultKeysConfig = configGroup.group(QStringLiteral("ResultKeysView"));
-    ui.resultTV->saveLayout(resultKeysConfig);
-
     configGroup.sync();
 }
 
+static QString guidanceText(LookupCertificatesDialog::QueryMode mode)
+{
+    switch (mode) {
+    default:
+        qCWarning(KLEOPATRA_LOG) << __func__ << "Unknown query mode:" << mode;
+        [[fallthrough]];
+    case LookupCertificatesDialog::AnyQuery:
+        return xi18nc("@info", "Enter a search term to search for matching certificates.");
+    case LookupCertificatesDialog::EmailQuery:
+        return xi18nc("@info", "Enter an email address to search for matching certificates.");
+    };
+}
+
+QValidator *LookupCertificatesDialog::Private::queryValidator()
+{
+    switch (queryMode) {
+    default:
+        qCWarning(KLEOPATRA_LOG) << __func__ << "Unknown query mode:" << queryMode;
+        [[fallthrough]];
+    case AnyQuery: {
+        if (!anyQueryValidator) {
+            // allow any query with at least one non-whitespace character
+            anyQueryValidator = new QRegularExpressionValidator{QRegularExpression{QStringLiteral(".*\\S+.*")}, q};
+        }
+        return anyQueryValidator;
+    }
+    case EmailQuery: {
+        if (!emailQueryValidator) {
+            // allow anything that looks remotely like an email address, i.e.
+            // anything with an '@' surrounded by non-whitespace characters
+            const QRegularExpression simpleEmailRegex{QStringLiteral(".*\\S+@\\S+.*")};
+            emailQueryValidator = new QRegularExpressionValidator{simpleEmailRegex, q};
+        }
+        return emailQueryValidator;
+    }
+    }
+}
+
+void LookupCertificatesDialog::Private::updateQueryMode()
+{
+    ui.guidanceLabel->setText(guidanceText(queryMode));
+    ui.findED->setValidator(queryValidator());
+}
+
 LookupCertificatesDialog::LookupCertificatesDialog(QWidget *p, Qt::WindowFlags f)
     : QDialog(p, f)
     , d(new Private(this))
@@ -269,10 +319,24 @@ LookupCertificatesDialog::~LookupCertificatesDialog()
     d->writeConfig();
 }
 
-void LookupCertificatesDialog::setCertificates(const std::vector<Key> &certs)
+void LookupCertificatesDialog::setQueryMode(QueryMode mode)
+{
+    d->queryMode = mode;
+    d->updateQueryMode();
+}
+
+LookupCertificatesDialog::QueryMode LookupCertificatesDialog::queryMode() const
+{
+    return d->queryMode;
+}
+
+void LookupCertificatesDialog::setCertificates(const std::vector<Key> &certs, const std::vector<Key::Origin> &origins)
 {
     d->ui.resultTV->view()->setFocus();
-    d->ui.resultTV->setKeys(certs);
+    d->ui.resultTV->setKeys(certs, origins);
+    if (certs.size() == 1) {
+        d->ui.resultTV->view()->setCurrentIndex(d->ui.resultTV->view()->model()->index(0, 0));
+    }
 }
 
 std::vector<Key> LookupCertificatesDialog::selectedCertificates() const
@@ -304,14 +368,21 @@ QString LookupCertificatesDialog::searchText() const
     return d->ui.findED->text();
 }
 
-void LookupCertificatesDialog::showInformation(const QString &message)
+void LookupCertificatesDialog::setOverlayText(const QString &text)
 {
-    d->ui.messageWidget->setText(message);
-    if (message.isEmpty()) {
-        d->ui.messageWidget->animatedHide();
+    if (text.isEmpty()) {
+        d->ui.overlay->hideOverlay();
     } else {
-        d->ui.messageWidget->animatedShow();
+        d->ui.overlay->setText(text);
+        d->ui.overlay->showOverlay();
     }
+    d->ui.selectAllPB->setEnabled(text.isEmpty());
+    d->ui.deselectAllPB->setEnabled(text.isEmpty());
+}
+
+QString LookupCertificatesDialog::overlayText() const
+{
+    return d->ui.overlay->text();
 }
 
 void LookupCertificatesDialog::accept()
@@ -335,7 +406,9 @@ void LookupCertificatesDialog::Private::enableDisableWidgets()
         return;
     }
 
-    ui.findPB->setEnabled(searchText().size() >= minimalSearchTextLength);
+    q->setOverlayText({});
+
+    ui.findPB->setEnabled(ui.findED->hasAcceptableInput());
 
     const int n = q->selectedCertificates().size();
 
