@@ -15,10 +15,7 @@
 
 #include <vector>
 
-namespace GpgME
-{
-class Key;
-}
+#include <gpgme++/key.h>
 
 namespace Kleo
 {
@@ -29,17 +26,27 @@ class LookupCertificatesDialog : public QDialog
 {
     Q_OBJECT
 public:
+    enum QueryMode {
+        AnyQuery, //< any query is allowed
+        EmailQuery, //< only email queries are allowed
+    };
+
     explicit LookupCertificatesDialog(QWidget *parent = nullptr, Qt::WindowFlags f = {});
     ~LookupCertificatesDialog() override;
 
-    void setCertificates(const std::vector<GpgME::Key> &certs);
+    void setQueryMode(QueryMode mode);
+    QueryMode queryMode() const;
+
+    void setCertificates(const std::vector<GpgME::Key> &certs, const std::vector<GpgME::Key::Origin> &origins);
     std::vector<GpgME::Key> selectedCertificates() const;
 
     void setPassive(bool passive);
     bool isPassive() const;
     void setSearchText(const QString &text);
     QString searchText() const;
-    void showInformation(const QString &message);
+
+    void setOverlayText(const QString &text);
+    QString overlayText() const;
 
 Q_SIGNALS:
     void searchTextChanged(const QString &text);
