@@ -70,9 +70,6 @@ public:
     const MainWindow *mainWindow() const;
     MainWindow *mainWindow();
 
-    const SysTrayIcon *sysTrayIcon() const;
-    SysTrayIcon *sysTrayIcon();
-
     void setIgnoreNewInstance(bool on);
     bool ignoreNewInstance() const;
     void toggleMainWindowVisibility();
@@ -86,10 +83,11 @@ public:
 public Q_SLOTS:
     void openOrRaiseMainWindow();
     void openOrRaiseConfigDialog();
+    void openOrRaiseGroupsConfigDialog(QWidget *parent);
 #ifndef QT_NO_SYSTEMTRAYICON
     void startMonitoringSmartCard();
-    void importCertificatesFromFile(const QStringList &files, GpgME::Protocol proto);
 #endif
+    void importCertificatesFromFile(const QStringList &files, GpgME::Protocol proto);
     void encryptFiles(const QStringList &files, GpgME::Protocol proto);
     void signFiles(const QStringList &files, GpgME::Protocol proto);
     void signEncryptFiles(const QStringList &files, GpgME::Protocol proto);
@@ -99,6 +97,8 @@ public Q_SLOTS:
     void checksumFiles(const QStringList &files, GpgME::Protocol /* unused */);
     void slotActivateRequested(const QStringList &arguments, const QString &workingDirectory);
 
+    void handleFiles(const QStringList &files, WId parentId = 0);
+
 Q_SIGNALS:
     /* Emitted from slotActivateRequested to enable setting the
      * correct exitValue */
