@@ -30,19 +30,23 @@
 #include "commands/selftestcommand.h"
 #include "commands/signencryptfilescommand.h"
 
-#include "conf/groupsconfigdialog.h"
-
 #include "utils/action_data.h"
 #include "utils/clipboardmenu.h"
 #include "utils/detail_p.h"
 #include "utils/filedialog.h"
 #include "utils/gui-helper.h"
+#include "utils/keyexportdraghandler.h"
+#include "utils/userinfo.h"
 
 #include <Libkleo/GnuPG>
 
 #include "dialogs/updatenotification.h"
 
+// needed for GPGME_VERSION_NUMBER
+#include <gpgme.h>
+
 #include "kleopatra_debug.h"
+#include <KAboutApplicationDialog>
 #include <KAboutData>
 #include <KActionCollection>
 #include <KActionMenu>
@@ -50,6 +54,7 @@
 #include <KConfigDialog>
 #include <KConfigGroup>
 #include <KEditToolBar>
+#include <KHelpMenu>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KShortcutsDialog>
@@ -89,6 +94,11 @@
 #include <Libkleo/SystemInfo>
 
 #include <KSharedConfig>
+
+#ifdef Q_OS_UNIX
+#include <KWaylandExtras>
+#endif
+
 #include <chrono>
 #include <vector>
 using namespace std::chrono_literals;
@@ -200,6 +210,12 @@ public:
 
     void closeAndQuit()
     {
+        if (Kleo::userIsElevated()) {
+            // For users running Kleo with elevated permissions on Windows we
+            // always quit the application to avoid some problems.
+            qApp->quit();
+        }
+
         const QString app = KAboutData::applicationData().displayName();
         const int rc = KMessageBox::questionTwoActionsCancel(q,
                                                              xi18n("<application>%1</application> may be used by other applications as a service.<nl/>"
@@ -209,11 +225,11 @@ public:
                                                              KStandardGuiItem_close(),
                                                              KStandardGuiItem_quit(),
                                                              KStandardGuiItem::cancel(),
-                                                             QLatin1String("really-quit-") + app.toLower());
+                                                             QLatin1StringView("really-quit-") + app.toLower());
         if (rc == KMessageBox::Cancel) {
             return;
         }
-        isQuitting = true;
+        isQuitting = rc == KMessageBox::ButtonCode::SecondaryAction;
         if (!q->close()) {
             return;
         }
@@ -281,11 +297,8 @@ public:
 
     void configureGroups()
     {
-        if (KConfigDialog::showDialog(GroupsConfigDialog::dialogName())) {
-            return;
-        }
-        KConfigDialog *dialog = new GroupsConfigDialog(q);
-        dialog->show();
+        // open groups config dialog as independent top-level window
+        KleopatraApplication::instance()->openOrRaiseGroupsConfigDialog(nullptr);
     }
 
     void showHandbook();
@@ -362,7 +375,25 @@ public:
 
     void restartDaemons()
     {
-        Kleo::killDaemons();
+        Kleo::restartGpgAgent();
+    }
+
+    void showAboutDialog()
+    {
+        // we show the About dialog ourselves so that we can pass up-to-date about data to it;
+        // KXmlGuiWindow takes a copy of the about data on creation and this copy might not
+        // contain the backend version information that's set by a background thread
+        if (!aboutDialog) {
+            qCDebug(KLEOPATRA_LOG) << __func__ << "Creating About dialog";
+            aboutDialog = new KAboutApplicationDialog(KAboutData::applicationData(), q);
+            aboutDialog->setAttribute(Qt::WA_DeleteOnClose);
+        }
+        if (aboutDialog->isMinimized()) {
+            qCDebug(KLEOPATRA_LOG) << __func__ << "Unminimizing About dialog";
+            aboutDialog->setWindowState((aboutDialog->windowState() & ~Qt::WindowMinimized) | Qt::WindowActive);
+        }
+        qCDebug(KLEOPATRA_LOG) << __func__ << "Showing About dialog";
+        aboutDialog->show();
     }
 
 private:
@@ -395,6 +426,7 @@ private:
     } ui;
     QAction *focusToClickSearchAction = nullptr;
     ClipboardMenu *clipboadMenu = nullptr;
+    QPointer<KAboutApplicationDialog> aboutDialog;
 };
 
 MainWindow::Private::UI::UI(MainWindow *q)
@@ -430,13 +462,17 @@ MainWindow::Private::Private(MainWindow *qq)
     KDAB_SET_OBJECT_NAME(controller);
 
     AbstractKeyListModel *flatModel = AbstractKeyListModel::createFlatKeyListModel(q);
-    flatModel->useKeyCache(true, KeyList::AllKeys);
     AbstractKeyListModel *hierarchicalModel = AbstractKeyListModel::createHierarchicalKeyListModel(q);
-    hierarchicalModel->useKeyCache(true, KeyList::AllKeys);
 
     KDAB_SET_OBJECT_NAME(flatModel);
     KDAB_SET_OBJECT_NAME(hierarchicalModel);
 
+#if GPGME_VERSION_NUMBER >= 0x011800 // 1.24.0
+    auto keyExportDragHandler = std::make_shared<KeyExportDragHandler>();
+    flatModel->setDragHandler(keyExportDragHandler);
+    hierarchicalModel->setDragHandler(keyExportDragHandler);
+#endif
+
     controller.setFlatModel(flatModel);
     controller.setHierarchicalModel(hierarchicalModel);
     controller.setTabWidget(ui.searchTab->tabWidget());
@@ -444,6 +480,15 @@ MainWindow::Private::Private(MainWindow *qq)
     ui.searchTab->tabWidget()->setFlatModel(flatModel);
     ui.searchTab->tabWidget()->setHierarchicalModel(hierarchicalModel);
 
+#ifdef Q_OS_UNIX
+    connect(KWaylandExtras::self(), &KWaylandExtras::windowExported, q, [this](const auto &window, const auto &token) {
+        if (window == q->windowHandle()) {
+            qputenv("PINENTRY_GEOM_HINT", QUrl::toPercentEncoding(token));
+        }
+    });
+    q->exportWindow();
+#endif
+
     setupActions();
 
     ui.stackWidget->setCurrentWidget(ui.searchTab);
@@ -458,6 +503,13 @@ MainWindow::Private::Private(MainWindow *qq)
 
     q->createGUI(QStringLiteral("kleopatra.rc"));
 
+    if (auto helpMenu = q->findChild<KHelpMenu *>()) {
+        qCDebug(KLEOPATRA_LOG) << "Hook into the help menu to show the About dialog ourselves";
+        connect(helpMenu, &KHelpMenu::showAboutApplication, q, [this]() {
+            showAboutDialog();
+        });
+    }
+
     // make toolbar buttons accessible by keyboard
     auto toolbar = q->findChild<KToolBar *>();
     if (toolbar) {
@@ -490,6 +542,17 @@ MainWindow::Private::Private(MainWindow *qq)
     if (KeyCache::instance()->initialized()) {
         keyListingDone();
     }
+
+    // delay setting the models to use the key cache so that the UI (including
+    // the "Loading certificate cache..." overlay) is shown before the
+    // blocking key cache initialization happens
+    QMetaObject::invokeMethod(
+        q,
+        [flatModel, hierarchicalModel]() {
+            flatModel->useKeyCache(true, KeyList::AllKeys);
+            hierarchicalModel->useKeyCache(true, KeyList::AllKeys);
+        },
+        Qt::QueuedConnection);
 }
 
 MainWindow::Private::~Private()
@@ -607,6 +670,18 @@ void MainWindow::Private::setupActions()
                 configureGroups();
             },
             QString(),
+        },
+        // Toolbar
+        {
+            "configure_groups_toolbar",
+            i18nc("@action:intoolbar", "Groups"),
+            QString(),
+            "group",
+            q,
+            [this](bool) {
+                configureGroups();
+            },
+            QString(),
         }};
 
     make_actions_from_data(action_data, coll);
@@ -647,6 +722,7 @@ void MainWindow::Private::setupActions()
                                                 "Please check with Gpg4win before translating this filename.",
                                                 "gpg4win-compendium-en.pdf"),
                                           QStringLiteral("../share/gpg4win"),
+                                          QUrl(),
                                           coll);
     coll->addAction(QStringLiteral("help_doc_compendium"), compendium);
 
@@ -658,6 +734,7 @@ void MainWindow::Private::setupActions()
                       i18n("&Quickguide"),
                       i18nc("Only available in German and English. Leave to English for other languages.", "encrypt_and_sign_gnupgvsd_en.pdf"),
                       QStringLiteral("../share/doc/gnupg-vsd"),
+                      QUrl(),
                       coll);
     coll->addAction(QStringLiteral("help_doc_quickguide"), quickguide);
 
@@ -666,6 +743,7 @@ void MainWindow::Private::setupActions()
                       i18n("&Password-based encryption"),
                       i18nc("Only available in German and English. Leave to English for other languages.", "symmetric_encryption_gnupgvsd_en.pdf"),
                       QStringLiteral("../share/doc/gnupg-vsd"),
+                      QUrl(),
                       coll);
     coll->addAction(QStringLiteral("help_doc_symenc"), symguide);
 
@@ -673,6 +751,7 @@ void MainWindow::Private::setupActions()
                                       i18n("&Group configuration"),
                                       i18nc("Only available in German and English. Leave to English for other languages.", "groupfeature_gnupgvsd_en.pdf"),
                                       QStringLiteral("../share/doc/gnupg-vsd"),
+                                      QUrl(),
                                       coll);
     coll->addAction(QStringLiteral("help_doc_groups"), groups);
 
@@ -682,6 +761,7 @@ void MainWindow::Private::setupActions()
                       i18n("&Mail encryption in Outlook"),
                       i18nc("Only available in German and English. Leave to English for other languages. Only shown on Windows.", "gpgol_outlook_addin_en.pdf"),
                       QStringLiteral("../share/doc/gnupg-vsd"),
+                      QUrl(),
                       coll);
     coll->addAction(QStringLiteral("help_doc_gpgol"), gpgol);
 #endif
@@ -692,6 +772,7 @@ void MainWindow::Private::setupActions()
                       i18n("&Certification Management"),
                       i18nc("Only available in German and English. Leave to English for other languages.", "certification_management_gnupgvsd_en.pdf"),
                       QStringLiteral("../share/doc/gnupg-vsd"),
+                      QUrl(),
                       coll);
     coll->addAction(QStringLiteral("help_doc_cert_management"), certmngmnt);
 
@@ -700,6 +781,7 @@ void MainWindow::Private::setupActions()
                       i18n("&Smartcard setup"),
                       i18nc("Only available in German and English. Leave to English for other languages.", "smartcard_setup_gnupgvsd_en.pdf"),
                       QStringLiteral("../share/doc/gnupg-vsd"),
+                      QUrl(),
                       coll);
     coll->addAction(QStringLiteral("help_doc_smartcard"), smartcard);
 
@@ -707,15 +789,26 @@ void MainWindow::Private::setupActions()
                                          i18n("GnuPG Command&line"),
                                          QStringLiteral("gnupg_manual_en.pdf"),
                                          QStringLiteral("../share/doc/gnupg-vsd"),
+                                         QUrl(QStringLiteral("https://gnupg.org/documentation/manuals/gnupg/")),
                                          coll);
     coll->addAction(QStringLiteral("help_doc_gnupg"), man_gnupg);
 
     /* The secops */
+    const auto approvalmanual =
+        new DocAction(QIcon::fromTheme(QStringLiteral("dvipdf")),
+                      i18n("Manual for VS-NfD approval (German)"),
+                      i18nc("Only available in German. Keep German file name for all languages", "Handbuch-Zulassung-gnupgvsd-v3.2.pdf"),
+                      QStringLiteral("../share/doc/gnupg-vsd"),
+                      QUrl(),
+                      coll);
+    coll->addAction(QStringLiteral("help_doc_approval_manual"), approvalmanual);
+
     const auto vsa10573 =
         new DocAction(QIcon::fromTheme(QStringLiteral("dvipdf")),
                       i18n("SecOps VSA-10573"),
                       i18nc("Only available in German and English. Leave to English for other languages.", "BSI-VSA-10573-ENG_secops-20220207.pdf"),
                       QStringLiteral("../share/doc/gnupg-vsd"),
+                      QUrl(),
                       coll);
     coll->addAction(QStringLiteral("help_doc_vsa10573"), vsa10573);
 
@@ -724,6 +817,7 @@ void MainWindow::Private::setupActions()
                       i18n("SecOps VSA-10584"),
                       i18nc("Only available in German and English. Leave to English for other languages.", "BSI-VSA-10584-ENG_secops-20220207.pdf"),
                       QStringLiteral("../share/doc/gnupg-vsd"),
+                      QUrl(),
                       coll);
     coll->addAction(QStringLiteral("help_doc_vsa10584"), vsa10584);
 
@@ -770,8 +864,9 @@ void MainWindow::closeEvent(QCloseEvent *e)
             setEnabled(true);
         }
     }
-    if (isQuitting || qApp->isSavingSession()) {
-        d->ui.searchTab->tabWidget()->saveViews(KSharedConfig::openConfig().data());
+    unexportWindow();
+    if (isQuitting || qApp->isSavingSession() || Kleo::userIsElevated()) {
+        d->ui.searchTab->tabWidget()->saveViews();
         KConfigGroup grp(KConfigGroup(KSharedConfig::openConfig(), autoSaveGroup()));
         saveMainWindowSettings(grp);
         e->accept();
@@ -785,7 +880,7 @@ void MainWindow::showEvent(QShowEvent *e)
 {
     KXmlGuiWindow::showEvent(e);
     if (d->firstShow) {
-        d->ui.searchTab->tabWidget()->loadViews(KSharedConfig::openConfig().data());
+        d->ui.searchTab->tabWidget()->loadViews(KSharedConfig::openStateConfig(), QStringLiteral("KeyList"));
         d->firstShow = false;
     }
 
@@ -851,42 +946,7 @@ void MainWindow::dropEvent(QDropEvent *e)
 
     const QStringList files = extract_local_files(e->mimeData());
 
-    const unsigned int classification = classify(files);
-
-    QMenu menu;
-
-    QAction *const signEncrypt = menu.addAction(i18n("Sign/Encrypt..."));
-    QAction *const decryptVerify = mayBeAnyMessageType(classification) ? menu.addAction(i18n("Decrypt/Verify...")) : nullptr;
-    if (signEncrypt || decryptVerify) {
-        menu.addSeparator();
-    }
-
-    QAction *const importCerts = mayBeAnyCertStoreType(classification) ? menu.addAction(i18n("Import Certificates")) : nullptr;
-    QAction *const importCRLs = mayBeCertificateRevocationList(classification) ? menu.addAction(i18n("Import CRLs")) : nullptr;
-    if (importCerts || importCRLs) {
-        menu.addSeparator();
-    }
-
-    if (!signEncrypt && !decryptVerify && !importCerts && !importCRLs) {
-        return;
-    }
-
-    menu.addAction(i18n("Cancel"));
-    const QAction *const chosen = menu.exec(mapToGlobal(e->position().toPoint()));
-
-    if (!chosen) {
-        return;
-    }
-
-    if (chosen == signEncrypt) {
-        d->createAndStart<SignEncryptFilesCommand>(files);
-    } else if (chosen == decryptVerify) {
-        d->createAndStart<DecryptVerifyFilesCommand>(files);
-    } else if (chosen == importCerts) {
-        d->createAndStart<ImportCertificateFromFileCommand>(files);
-    } else if (chosen == importCRLs) {
-        d->createAndStart<ImportCrlCommand>(files);
-    }
+    KleopatraApplication::instance()->handleFiles(files);
 
     e->accept();
 }
@@ -905,5 +965,25 @@ void MainWindow::saveProperties(KConfigGroup &cg)
     cg.writeEntry("hidden", isHidden());
 }
 
+void MainWindow::exportWindow()
+{
+#ifdef Q_OS_UNIX
+    (void)winId(); // Ensures that windowHandle() returns the window
+    KWaylandExtras::self()->exportWindow(windowHandle());
+#endif
+}
+
+void MainWindow::unexportWindow()
+{
+#ifdef Q_OS_UNIX
+    KWaylandExtras::self()->unexportWindow(windowHandle());
+#endif
+}
+
+KeyListController *MainWindow::keyListController()
+{
+    return &d->controller;
+}
+
 #include "mainwindow.moc"
 #include "moc_mainwindow.cpp"
