@@ -14,6 +14,8 @@
 
 #include "card.h"
 
+#include <gpgme++/global.h>
+
 #include <memory>
 #include <vector>
 
@@ -49,11 +51,11 @@ public:
 
     Card::Status cardStatus(unsigned int slot) const;
     std::string firstCardWithNullPin() const;
-    bool anyCardCanLearnKeys() const;
 
     std::vector<std::shared_ptr<Card>> getCards() const;
 
     std::shared_ptr<Card> getCard(const std::string &serialNumber, const std::string &appName) const;
+    std::shared_ptr<Card> getCardWithKeyRef(const std::string &serialNumber, const std::string &keyRef) const;
 
     template<typename T>
     std::shared_ptr<T> getCard(const std::string &serialNumber) const
@@ -69,18 +71,23 @@ public:
 public Q_SLOTS:
     void updateStatus();
     void updateCard(const std::string &serialNumber, const std::string &appName);
+    void learnCards(GpgME::Protocol protocol);
     void startMonitoring();
 
 Q_SIGNALS:
     void firstCardWithNullPinChanged(const std::string &serialNumber);
-    void anyCardCanLearnKeysChanged(bool);
     void cardAdded(const std::string &serialNumber, const std::string &appName);
     void cardChanged(const std::string &serialNumber, const std::string &appName);
     void cardRemoved(const std::string &serialNumber, const std::string &appName);
     void updateFinished();
+    void startingLearnCards(GpgME::Protocol protocol);
+    void cardsLearned(GpgME::Protocol protocol);
     void startOfGpgAgentRequested();
 
 private:
+    void onStartingLearnCards(GpgME::Protocol protocol);
+    void onCardsLearned(GpgME::Protocol protocol);
+
     class Private;
     std::shared_ptr<Private> d;
 };
