@@ -9,6 +9,8 @@
 
 #pragma once
 
+#include <KSharedConfig>
+
 #include <QWidget>
 
 #include <memory>
@@ -20,7 +22,6 @@ class QAbstractItemView;
 
 class KConfigGroup;
 class KActionCollection;
-class KConfig;
 
 namespace Kleo
 {
@@ -35,6 +36,12 @@ class TabWidget : public QWidget
 {
     Q_OBJECT
 public:
+    enum Option {
+        ShowKeys = 0x00,
+        ShowUserIDs = 0x01,
+    };
+    Q_DECLARE_FLAGS(Options, Option)
+
     explicit TabWidget(QWidget *parent = nullptr, Qt::WindowFlags f = {});
     ~TabWidget() override;
 
@@ -44,12 +51,12 @@ public:
     AbstractKeyListModel *hierarchicalModel() const;
 
     QAbstractItemView *addView(const QString &title = QString(), const QString &keyFilterID = QString(), const QString &searchString = QString());
-    QAbstractItemView *addView(const KConfigGroup &group);
+    QAbstractItemView *addView(const KConfigGroup &group, Options options);
     QAbstractItemView *
     addTemporaryView(const QString &title = QString(), AbstractKeyListSortFilterProxyModel *proxy = nullptr, const QString &tabToolTip = QString());
 
-    void loadViews(const KConfig *cfg);
-    void saveViews(KConfig *cfg) const;
+    void loadViews(const KSharedConfig::Ptr &config, const QString &configKeys, Options options = ShowKeys);
+    void saveViews();
 
     std::vector<QAbstractItemView *> views() const;
     QAbstractItemView *currentView() const;
